import java.util.*;
import java.io.*;

/**************************************************************
 * This program takes bowtie-aligned paired reads and runs EM to
 * learn edge weights on a new-format PSG.
 **************************************************************/

public class AlignPaired {
	
    public static SpliceGraph s;
	
	public static HashMap<String,ArrayList<Integer>> first_ids;
	public static HashMap<String,ArrayList<Integer>> second_ids;
    public static String[][] reads;
	public static ArrayList[] align;
	public static ArrayList[] confidence;
	public static ArrayList[] lengths;
	public static double[] c1;
	public static double[] c2;
	public static int fasta_readlen;
	
	public static ArrayList<Integer[]> align1;
	public static ArrayList<Integer[]> align2;
    
    public static void main(String[] args) {
		if (args.length != 4 && args.length != 7) {
			System.out.println("usage: Align <psg> <bowtie.1.align> <bowtie.2.align> <FASTA readlen> [<mean> <stdev> <beta>]");
			System.exit(0);
		}
		
		s = new SpliceGraph(args[0]);
		
		String filename1 = args[1];
		String filename2 = args[2];
		fasta_readlen = Integer.parseInt(args[3]);
		//System.err.println("Processing Reads...");
		process_reads(filename1,filename2);
		//System.err.println("Reads processed.");
		
		try {
			s.setMean(Integer.parseInt(args[4]));
			s.setStdev(Double.parseDouble(args[5]));
			s.setBeta(Double.parseDouble(args[6]));
		} catch (ArrayIndexOutOfBoundsException e) {
			// do nothing, this is okay
		}
		catch (NumberFormatException nfe) {
			System.out.print("Error: Mean should be given as an integer, ");
			System.out.println("Stdev should be given as a double.");
			System.exit(0);
		}
		
		// checking the edge case where we don't have any aligned reads
		if (reads.length == 0) {
			double ll = 0.0;
			//System.out.println("\tNumber of null reads: 0");
			System.out.println("\tNumber of reads: 0");
			s.emOutput(ll,-1);
			
			System.out.println();
			System.out.println("Current log likelihood: "+ll);
			System.out.println("Converged after 0 iterations");
			System.out.println("Log likelihood diff: 0");
			System.out.println("Max edge difference: 0.0");
			System.out.println("L1 norm: 0.0");
			System.out.println("L2 norm: 0.0");
			return;
		}
		
		// so if we're here, then we definitely have reads
		System.out.println("\tNumber of reads: "+reads.length);
		
		double[] initial = s.getEdgeWeights();
		s.resetEdgeWeights();
		int[][] length = Utility.IntArray(lengths);
		
		/*
		 for (int i=0; i<align.length; i++) {
		 for (int j=0; j<align[i].size(); j++) {
		 Integer[] temp = (Integer[]) align[i].get(j);
		 System.err.print("[");
		 for (int k=0; k<temp.length; k++) {
		 System.err.print(temp[k]);
		 if (k < temp.length-1) {
		 System.err.print(", ");
		 }
		 }
		 System.err.print("]");
		 }
		 System.err.println();
		 }
		 // */
		/*
		 for (int i=0; i<confidence.length; i++) {
		 for (int j=0; j<confidence[i].size(); j++) {
		 System.err.print(confidence[i].get(j));
		 if (j < confidence[i].size()-1) {
		 System.err.print(", ");
		 }
		 }
		 System.err.println();
		 }
		 // */
		/*
		 for (int i=0; i<length.length; i++) {
		 for (int j=0; j<length[i].length; j++) {
		 System.err.print(length[i][j]);
		 if (j < length[i].length-1) {
		 System.err.print(", ");
		 }
		 }
		 System.err.println();
		 }
		 // */
		
		s.learnEM(reads,align,confidence,1,length);
		
		double[] a = s.getEdgeWeights();
		double result = 0;
		for (int i=0; i<initial.length; i++) {
			result += Math.abs(a[i]-initial[i]);
		}
		System.out.println("L1 norm: "+result);
		result = 0;
		for (int i=0; i<initial.length; i++) {
			result += Math.pow(a[i]-initial[i],2);
		}
		result = Math.sqrt(result);
		System.out.println("L2 norm: "+result);
    }
	
	// reads information from a bowtie alignment file
    private static void process_reads(String filename1, String filename2) {
		// initialize the temporary variables
		ArrayList<Integer> start1 = new ArrayList<Integer>();
		ArrayList<Integer> start2 = new ArrayList<Integer>();
		ArrayList<String> read1 = new ArrayList<String>();
		ArrayList<String> read2 = new ArrayList<String>();
		ArrayList<String> alig1 = new ArrayList<String>();
		ArrayList<String> alig2 = new ArrayList<String>();
		ArrayList<String> which1 = new ArrayList<String>();
		ArrayList<Integer> length1 = new ArrayList<Integer>();
		ArrayList<Integer> length2 = new ArrayList<Integer>();
		
		ArrayList<String> mismatch = new ArrayList<String>();
		ArrayList<String> quality = new ArrayList<String>();
		
		// initialize the global variables
		reads = null;
		align = null;
		confidence = null;
		c1 = null;
		c2 = null;
		
		try {
			Scanner r = new Scanner(new File(filename1));
			while (r.hasNextLine()) {
				String[] components = r.nextLine().split("\t");
				which1.add(components[0]);
				length1.add(components[0].length());
				alig1.add(components[2]);
				start1.add(Integer.decode(components[3]));
				read1.add(components[4]);
				quality.add(components[5]);
				if (components.length == 8) {
					mismatch.add(components[7]);
				}
				else {
					mismatch.add("x");
				}
			}
		}
		catch (IOException ioe) {
			System.out.println("Alignment file "+filename1+" contains an error.");
			System.exit(0);
		}
		c1 = compute_quality(quality,mismatch);
		
		first_ids = new HashMap<String,ArrayList<Integer>>(which1.size());
		for (int i=0; i<which1.size(); i++) {
			if (first_ids.get(which1.get(i)) != null) {
				first_ids.get(which1.get(i)).add(i);
			}
			else {
				ArrayList<Integer> t = new ArrayList<Integer>();
				t.add(i);
				first_ids.put(which1.get(i),t);
			}
		}
		
		mismatch = new ArrayList<String>();
		quality = new ArrayList<String>();
		int index = 0;
		
		second_ids = new HashMap<String,ArrayList<Integer>>(which1.size());
		
		try {
			Scanner r = new Scanner(new File(filename2));
			while (r.hasNextLine()) {
				String[] components = r.nextLine().split("\t");
				if (second_ids.get(components[0]) != null) {
					second_ids.get(components[0]).add(index);
				}
				else {
					ArrayList<Integer> t = new ArrayList<Integer>();
					t.add(index);
					second_ids.put(components[0],t);
				}
				length2.add(components[0].length());
				alig2.add(components[2]);
				start2.add(Integer.decode(components[3]));
				read2.add(components[4]);
				quality.add(components[5]);
				if (components.length == 8) {
					mismatch.add(components[7]);
				}
				else {
					mismatch.add("x");
				}
				index++;
			}
		}
		catch (IOException ioe) {
			System.out.println("Alignment file "+filename2+" contains an error.");
			System.exit(0);
		}
		c2 = compute_quality(quality,mismatch);
		//System.err.println("\tQuality computed.");
		
		reads = getReads(which1,read1,read2);
		//System.err.println("\tReads got.");
		
		if (reads.length > 0) {
			process_align(which1,alig1,alig2,start1,start2,length1,length2);
			//System.err.println("\tAlignments processed.");
		}
    }
	
	// retrieves the reads from the read arraylist
	private static String[][] getReads(ArrayList<String> r1, ArrayList<String> s1, ArrayList<String> s2) {
		String[][] result = new String[count_unique(r1)][2];
		String id = null;
		int index = 0;
		HashMap<String,Integer> used_ids = new HashMap<String,Integer>();
		
		for (int i=0; i<r1.size(); i++) {
			if (!used_ids.containsKey(r1.get(i))) {
				id = r1.get(i);
				used_ids.put(id,0);
				//result[index][0] = s1.get(i);
				id = id.substring(0,id.length()-1);
				id += "2";
				ArrayList<Integer> indices = second_ids.get(id);
				if (indices != null) {
					for (int j=0; j<indices.size(); j++) {
						result[index][0] = s1.get(i);
						result[index][1] = s2.get(indices.get(j));
					}
				}
				if (index >= result.length || result[index][0] != null) {
					index++;
				}
			}
		}
		return trim(result);
    }
    
    // Computes confidence based on match/mismatch; quality of read
    private static double[] compute_quality(ArrayList<String> q, ArrayList<String> m) {
		double[] result = new double[q.size()];
		for (int i=0; i<q.size(); i++) {
			result[i] = 1; // initializing for multiplication
			// check for mismatches
			int[] positions = {-1};
			if (!m.get(i).equals("x")) {
				// process mismatches
				String[] mismatch = m.get(i).split(",");
				positions = new int[mismatch.length];
				for (int j=0; j<mismatch.length; j++) {
					// get position of mismatch - part before : should be position
					String[] temp = mismatch[j].split(":");
					positions[j] = Integer.parseInt(temp[0]);
				}
			}
			// process quality scores
			int index = 0; // for positions[], assume listed in increasing order
			for (int j=0; j<q.get(i).length(); j++) {
				int quality = (int) q.get(i).charAt(j);
				quality -= 33; // because we have a +33 offset
				if (index >= positions.length || j != positions[index]) {
					// not a mismatch
					result[i] *= 1-Math.pow(10,(-quality/10.0));
				}
				else {
					// mismatch
					result[i] *= (1/3.0)*Math.pow(10,(-quality/10.0));
					index++;
				}
			}
		}
		return result;
    }
    
    // Makes an id/alignment index hash so that we don't have to search
    private static HashMap<String,String> makeAligHash(ArrayList<String> id, ArrayList<String> alig) {
    	HashMap<String,String> result = new HashMap<String,String>();
    	for (int i=0; i<id.size(); i++) {
    		if (!result.containsKey(id.get(i))) {
    			result.put(id.get(i), i+"");
    		}
    		else {
    			String index_list = result.get(id.get(i));
    			index_list += ","+i;
    			result.put(id.get(i),index_list);
    		}
    	}
    	return result;
    }
	
	// consolidates reads into a single string array with only one occurrence of each read
    private static String[][] process_reads(ArrayList<String> r1, ArrayList<String> r2, 
											ArrayList<String> id1, ArrayList<String>id2) {
		int num1 = count_unique(id1);
		int num2 = count_unique(id2);
		HashMap<String,String> k = makeHash(r2,id2);
		String[][] reads = new String[Math.min(num1,num2)][2];
		String id = null;
		int index = 0;
		HashMap<String,Integer> used_ids = new HashMap<String,Integer>();
		for (int i=0; i<id1.size(); i++) {
			if (!used_ids.containsKey(id1.get(i))) {
				id = id1.get(i);
				used_ids.put(id,1);
				String part2 = id.substring(0,id.length()-1);
				part2 += "2";
				if (k.containsKey(part2)) {
					String read = k.get(part2);
					reads[index][0] = r1.get(i);
					reads[index][1] = read;
					index++;
				}
			}
		}
		return trim(reads);
    }
	
    // returns a HashMap verson of a read/id pair
    // if there is more than one copy of a read, key is number of copies
    // if only one copy of a read, key is read text
    private static HashMap<String,String> makeHash(ArrayList<String> read, ArrayList<String> id) {
    	HashMap<String,String> result = new HashMap<String,String>();
    	for (int i=0; i<id.size(); i++) {
    		if (!result.containsKey(id.get(i))) {
    			result.put(id.get(i), read.get(i));
    		}
    	}
    	return result;
    }
    
    private static String[][] trim(String[][] to_trim) {
		int i=0;
		for (i=0; i<to_trim.length; i++) {
			if (to_trim[i][1] == null) {
				break;
			}
		}
		String[][] result = new String[i][];
		for (i=0; i<result.length; i++) {
			result[i] = to_trim[i];
		}
		return result;
    }
    
    private static int count_unique(ArrayList<String> r) {
		if (r.size() == 0) { return 0; }
		HashMap<String,Integer> unique = new HashMap<String,Integer>();
		for (int i=0; i<r.size(); i++) {
			if (!unique.containsKey(r.get(i))) {
				unique.put(r.get(i),1);
			}
		}
		return unique.size();
    }
	
    private static String[] getReads(int[] r, ArrayList<String> s) {
		String[] result = new String[r[r.length-1]+1];
		for (int i=0; i<s.size(); i++) {
			result[r[i]] = s.get(i);
		}
		return result;
    }
    
    private static int[] count_mismatch(ArrayList<String> m) {
		int[] result = new int[m.size()];
		int k = 0;
		for (int i=0; i<m.size(); i++) {
			if (!m.get(i).equals("x")) {
				String[] n = m.get(i).split(",");
				result[i] = n.length;
				k++;
			}
		}
		System.out.println(k+" mismatched alignments");
		return result;
    }
    
    private static ArrayList[] getAlignments(ArrayList<String> a, String[] r) {
		ArrayList[] result = new ArrayList[r.length];
		for (int i=0; i<result.length; i++) {
			result[i] = new ArrayList<Integer[]>();
		}
		for (int i=0; i<a.size(); i++) {
			String[] align = a.get(i).substring(1).split(":");
			Integer[] c = new Integer[align.length];
			for (int j=0; j<align.length; j++) {
				c[j] = Integer.decode(align[j]);
			}
			if (!contains(result[i],c)) {
				result[i].add(c);
			}
		}
		return result;
    }
	
    private static boolean contains(ArrayList<String> a, String b) {
		boolean retval = false;
		for (int i=0; i<a.size(); i++) {
			if (a.get(i).equals(b)) {
				retval = true;
			}
		}
		return retval;
    }
	
    private static boolean contains(ArrayList<Integer[]> a, Integer[] b) {
		boolean retval = false;
		for (int i=0; i<a.size(); i++) {
			if (a.get(i).length == b.length) {
				boolean verify = true;
				for (int j=0; j<a.get(i).length; j++) {
					if (!a.get(i)[j].equals(b[j])) {
						verify = false;
					}
				}
				if (verify) {
					retval = true;
					break;
				}
			}
		}
		return retval;
    }
    
    private static ArrayList<Integer> indexOf(ArrayList<Integer[]> a, Integer[] b) {
		ArrayList<Integer> retval = new ArrayList<Integer>();
		for (int i=0; i<a.size(); i++) {
			if (a.get(i).length == b.length) {
				boolean verify = true;
				for (int j=0; j<a.get(i).length; j++) {
					if (!a.get(i)[j].equals(b[j])) {
						verify = false;
					}
				}
				if (verify) {
					retval.add(i);
				}
			}
		}
		return retval;
    }
    
    public static void initialize_align(ArrayList<Integer[]> which, ArrayList<String> al) {
    	for (int i=0; i<al.size(); i++) {
    		which.add(new Integer[1]);
    	}
    }
	
	// Trim down the alignments to deal with the fasta_readlen issues
	private static void trim(ArrayList<String> id1, ArrayList<String> al1, ArrayList<String> al2,
							 ArrayList<Integer> st1, ArrayList<Integer> st2, ArrayList<Integer> len1,
							 ArrayList<Integer> len2) {
		HashMap<String,Integer> used_1 = new HashMap<String,Integer>();
		align1 = new ArrayList<Integer[]>(al1.size());
		align2 = new ArrayList<Integer[]>(al2.size());
		initialize_align(align1, al1);
		initialize_align(align2, al2);
		
		// first: trim alignments
		int x = 0;	// index into first/second_paths/start
		for (int i=0; i<id1.size(); i++) {
			String id = id1.get(i);
			if (!used_1.containsKey(id)) {
				used_1.put(id,1);
				ArrayList<Integer> index1 = first_ids.get(id);
				
				for (int j=0; j<index1.size(); j++) {
					// get path
					int index = index1.get(j).intValue();
					String[] match = al1.get(index).substring(1).split(":");
					Integer[] r = new Integer[match.length];
					for (int k=0; k<match.length; k++) {
						r[k] = Integer.decode(match[k]);
					}
					Integer[] t = null;
					t = adjust_alignment(r,st1.get(index),len1.get(index));
					align1.set(index,t);
					
					// get new start position
					int k=0;
					int new_st1 = st1.get(index);
					if (r[k].intValue() != t[0].intValue()) {
						new_st1 -= Math.min(s.getSegment(r[k].intValue()).length(),fasta_readlen-1);
						k++;
					}
					else {
						if (r.length == 1) {
							// we're fine, only aligned to one exon anyway
						}
						else {
							// it's an intron-bridging alignment, add the rest of the segment
							if (s.getSegment(r[k].intValue()).length()-(fasta_readlen-1) > 0) {
								new_st1 += s.getSegment(r[k].intValue()).length() - (fasta_readlen - 1);
							}
						}
					}
					while (r[k].intValue() != t[0].intValue() && new_st1 > 0) {
						new_st1 -= s.getSegment(r[k].intValue()).length();
						k++;
					}
					st1.set(index,new_st1);
				}
				
				id = id.substring(0,id.length()-1);
				id += "2";
				ArrayList<Integer> index2 = second_ids.get(id);
				
				if (index2 != null) {
					for (int j=0; j<index2.size(); j++) {
						// get path
						int index = index2.get(j).intValue(); // this is not the READ index
						String[] match = al2.get(index).substring(1).split(":");
						Integer[] r = new Integer[match.length];
						for (int k=0; k<match.length; k++) {
							r[k] = Integer.decode(match[k]);
						}
						Integer[] t = null;
						t = adjust_alignment(r,st2.get(index),len2.get(index));
						align2.set(index,t);
						
						// get new start position
						int k=0;
						int new_st2 = st2.get(index);
						if (r[k].intValue() != t[0].intValue()) {
							new_st2 -= Math.min(s.getSegment(r[k].intValue()).length(),fasta_readlen-1);
							k++;
						}
						else {
							if (r.length == 1) {
								// we're fine, only aligned to one exon anyway
							}
							else {
								// it's an intron-bridging alignment, add the rest of the segment
								if (s.getSegment(r[k].intValue()).length()-(fasta_readlen-1) > 0) {
									new_st2 += s.getSegment(r[k].intValue()).length() - (fasta_readlen - 1);
								}
							}
						}
						while (r[k].intValue() != t[0].intValue() && new_st2 > 0) {
							new_st2 -= s.getSegment(r[k].intValue()).length();
							k++;
						}
						st2.set(index,new_st2);
					}
					x++;
				}
			}
		}		
	}
	
	// third time's the charm
	//*
	private static void process_align(ArrayList<String> id1,
									  ArrayList<String> al1, ArrayList<String> al2,
									  ArrayList<Integer> st1, ArrayList<Integer> st2,
									  ArrayList<Integer> len1, ArrayList<Integer> len2) {
		// initialize everything we'll need for the alignments
		align = new ArrayList[reads.length];
		confidence = new ArrayList[reads.length];
		lengths = new ArrayList[reads.length];
		
		// first: trim alignments
		trim(id1,al1,al2,st1,st2,len1,len2);
		
		// second: remove duplicates
		// after this, pairs will have the same index
		
		ArrayList[] first_paths = new ArrayList[first_ids.size()];
		ArrayList[] first_start = new ArrayList[first_ids.size()];
		ArrayList[] first_conf = new ArrayList[first_ids.size()];
		ArrayList[] second_paths = new ArrayList[first_ids.size()];
		ArrayList[] second_start = new ArrayList[first_ids.size()];
		ArrayList[] second_conf = new ArrayList[first_ids.size()];
		HashMap<String,Integer> used_ids = new HashMap<String,Integer>();
		int x = 0;
		int index = -1;
		
		for (int i=0; i<id1.size(); i++) {
			String id = id1.get(i);
			if (used_ids.containsKey(id)) {
				index = used_ids.get(id).intValue();
			}
			else {
				used_ids.put(id1.get(i),x);
				index = x++;
				first_paths[index] = new ArrayList<Integer[]>();
				first_start[index] = new ArrayList<Integer>();
				first_conf[index] = new ArrayList<Double>();
				second_paths[index] = new ArrayList<Integer[]>();
				second_start[index] = new ArrayList<Integer>();
				second_conf[index] = new ArrayList<Double>();
			}
			
			if (!contains(first_paths[index],align1.get(i))) {
				// no problems, add everything
				first_paths[index].add(align1.get(i));
				first_start[index].add(st1.get(i));
				first_conf[index].add(c1[i]);
			}
			else {
				ArrayList<Integer> temp = indexOf(first_paths[index],align1.get(i));
				boolean contained = false;
				for (int j=0; j<temp.size(); j++) {
					if (first_start[index].get(temp.get(j).intValue()).equals(st1.get(i))) {
						contained = true;
						break;
					}
				}
				if (!contained) {
					// different alignment to same path
					first_paths[index].add(align1.get(i));
					first_start[index].add(st1.get(i));
					first_conf[index].add(c1[i]);
				}
			}
			
			id = id.substring(0,id.length()-1);
			id += "2";
			ArrayList<Integer> id2 = second_ids.get(id);
			if (id2 != null) {
				for (int j=0; j<id2.size(); j++) {
					if (!contains(second_paths[index],align2.get(id2.get(j).intValue()))) {
						// no problems, add everything
						second_paths[index].add(align2.get(id2.get(j).intValue()));
						second_start[index].add(st2.get(id2.get(j).intValue()));
						second_conf[index].add(c2[id2.get(j).intValue()]);
					}
					else {
						ArrayList<Integer> temp = indexOf(second_paths[index],align2.get(id2.get(j).intValue()));
						boolean contained = false;
						for (int k=0; k<temp.size(); k++) {
							if (second_start[index].get(temp.get(k).intValue()).equals(st2.get(id2.get(j).intValue()))) {
								contained = true;
								break;
							}
						}
						if (!contained) {
							// different alignment to same path
							second_paths[index].add(align2.get(id2.get(j).intValue()));
							second_start[index].add(st2.get(id2.get(j).intValue()));
							second_conf[index].add(c2[id2.get(j).intValue()]);
						}
					}
				}
			}
		}
		
		// third: find paths
		index = 0;
		for (int i=0; i<first_paths.length; i++) {
			if (second_paths[i].size() > 0) {
				ArrayList[] paths = s.find_paths(first_paths[i],second_paths[i],first_conf[i],
												 second_conf[i],first_start[i],second_start[i],
												 reads[index][1].length());
				if (paths != null) {
					align[index] = paths[0];
					confidence[index] = paths[1];
					lengths[index] = paths[2];
					index++;
				}
			}
		}
	}
	// */
    
	// deprecated and ugly
	/*
	 private static void process_align(ArrayList<String> id1, 
	 ArrayList<String> al1, ArrayList<String> al2,
	 ArrayList<Integer> st1, ArrayList<Integer> st2) {
	 align = new ArrayList[reads.length];
	 confidence = new ArrayList[reads.length];
	 lengths = new ArrayList[reads.length];
	 
	 int index = 0;
	 int old_ind = -1;
	 
	 // one for each of the paired reads
	 ArrayList<Integer[]> matches1 = new ArrayList<Integer[]>();
	 ArrayList<Double> quality1 = new ArrayList<Double>();
	 ArrayList<Integer> pos1 = new ArrayList<Integer>();
	 ArrayList<Integer[]> matches2 = new ArrayList<Integer[]>();
	 ArrayList<Double> quality2 = new ArrayList<Double>();
	 ArrayList<Integer> pos2 = new ArrayList<Integer>();
	 
	 HashMap<String,Integer> used_ids = new HashMap<String,Integer>();
	 
	 for (int i=0; i<id1.size(); i++) {
	 // initialize everything
	 if (index == 155) {
	 System.out.println();
	 }
	 String id = id1.get(i);
	 if (used_ids.containsKey(id)) {
	 old_ind = used_ids.get(id).intValue();
	 }
	 
	 id = id.substring(0,id.length()-1);
	 id += "2";
	 ArrayList<Integer> id2 = second_ids.get(id);
	 
	 if (id2 != null) {
	 matches1 = new ArrayList<Integer[]>();
	 quality1 = new ArrayList<Double>();
	 pos1 = new ArrayList<Integer>();
	 
	 matches2 = new ArrayList<Integer[]>();
	 quality2 = new ArrayList<Double>();
	 pos2 = new ArrayList<Integer>();
	 
	 // get first read alignment and trim it
	 String[] match = al1.get(i).substring(1).split(":");
	 Integer[] r = new Integer[match.length];
	 for (int j=0; j<match.length; j++) {
	 r[j] = Integer.decode(match[j]);
	 }
	 Integer[] t = null;
	 if (index >= reads.length) {
	 t = adjust_alignment(r,st1.get(i).intValue(),reads[0][0].length());
	 }
	 else if (old_ind == -1) {
	 t = adjust_alignment(r,st1.get(i).intValue(),reads[index][0].length());
	 }
	 else {
	 t = adjust_alignment(r,st1.get(i).intValue(),reads[old_ind][0].length());
	 }
	 int k=0;
	 int new_st1 = st1.get(i);
	 if (r[k].intValue() != t[0].intValue()) {
	 new_st1 -= Math.min(s.getSegment(r[k].intValue()).length(),fasta_readlen-1);
	 k++;
	 }
	 else {
	 if (r.length == 1) {
	 // we're fine, only aligned to one exon anyway
	 }
	 else {
	 // it's an intron-bridging alignment, add the rest of the segment in
	 // TODO: if necessary!
	 if (s.getSegment(r[k].intValue()).length()-(fasta_readlen-1) > 0) {
	 new_st1 += s.getSegment(r[k].intValue()).length() - (fasta_readlen - 1);
	 }
	 }
	 }
	 while (r[k].intValue() != t[0].intValue() && new_st1 > 0) {
	 new_st1 -= s.getSegment(r[k].intValue()).length();
	 k++;
	 }
	 
	 if (!contains(matches1,t)) {
	 matches1.add(t);
	 quality1.add(c1[i]);
	 pos1.add(new_st1);
	 }
	 else {
	 ArrayList<Integer> x = indexOf(matches1,t);
	 boolean contained = false;
	 for (int y=0; y<x.size(); y++) {
	 if (pos1.get(x.get(y)).equals(new_st1)) {
	 contained = true;
	 }
	 }
	 if (!contained) {
	 matches1.add(t);
	 quality1.add(c1[i]);
	 pos1.add(new_st1);
	 }
	 }
	 
	 for (int j=0; j<id2.size(); j++) {
	 // we have a second half of this read
	 // get its alignment and trim it
	 int ind2 = id2.get(j);
	 String[] m = al2.get(ind2).substring(1).split(":");
	 r = new Integer[m.length];
	 for (k=0; k<m.length; k++) {
	 r[k] = Integer.decode(m[k]);
	 }
	 
	 if (old_ind == -1) {
	 t = adjust_alignment(r,st2.get(ind2).intValue(),reads[index][1].length());
	 }
	 else {
	 t = adjust_alignment(r,st2.get(ind2).intValue(),reads[old_ind][1].length());
	 }
	 k=0;
	 int new_st2 = st2.get(ind2);
	 if (r[k].intValue() != t[0].intValue()) {
	 new_st2 -= Math.min(s.getSegment(r[k].intValue()).length(),fasta_readlen-1);
	 k++;
	 }
	 else {
	 if (r.length == 1) {
	 // we're fine, only aligned to one exon anyway
	 }
	 else {
	 // it's an intron-bridging alignment, add the rest of the segment in
	 if (s.getSegment(r[k].intValue()).length()-(fasta_readlen-1) > 0) {
	 new_st2 += s.getSegment(r[k].intValue()).length() - (fasta_readlen - 1);
	 }
	 }
	 }
	 while (r[k].intValue() != t[0].intValue()) {
	 new_st2 -= s.getSegment(r[k].intValue()).length();
	 k++;
	 }
	 r = t;
	 
	 if (!contains(matches2,t)) {
	 matches2.add(t);
	 quality2.add(c2[ind2]);
	 pos2.add(new_st2);
	 }
	 else {
	 ArrayList<Integer> x = indexOf(matches2,t);
	 boolean contained = false;
	 for (int y=0; y<x.size(); y++) {
	 if (pos2.get(x.get(y)).equals(new_st2)) {
	 contained = true;
	 }
	 }
	 if (!contained) {
	 matches2.add(t);
	 quality2.add(c2[ind2]);
	 pos2.add(new_st2);
	 }
	 }
	 }
	 ArrayList[] paths = null;
	 if (index >= reads.length) {
	 paths = s.find_paths(matches1,matches2,quality1,quality2,pos1,pos2,reads[0][1].length());
	 }
	 else if (old_ind == -1) {
	 paths = s.find_paths(matches1,matches2,quality1,quality2,pos1,pos2,reads[index][1].length());
	 }
	 else {
	 paths = s.find_paths(matches1,matches2,quality1,quality2,pos1,pos2,
	 reads[old_ind][1].length());
	 }
	 
	 if (old_ind == -1 && paths != null) {
	 //System.out.print(index + " ");
	 used_ids.put(id1.get(i), index);
	 align[index] = paths[0];
	 confidence[index] = paths[1];
	 lengths[index] = paths[2];
	 index++;
	 }
	 else if (paths != null) {
	 // TODO: Check if this alignment has already been added to old_ind
	 for (int j=0; j<paths[0].size(); j++) {
	 ArrayList<Integer> indices = indexOf(align[old_ind],(Integer[])paths[0].get(j));
	 boolean alreadyThere = false;
	 if (indices.size() > 0) {
	 for (int m=0; m<indices.size(); m++) {
	 if (lengths[old_ind].get(indices.get(m)).equals(paths[2].get(j))) {
	 alreadyThere = true;
	 }
	 }
	 }
	 if (!alreadyThere) {
	 align[old_ind].add(paths[0].get(j));
	 confidence[old_ind].add(paths[1].get(j));
	 lengths[old_ind].add(paths[2].get(j));
	 }
	 }
	 //align[old_ind] = matches;
	 //confidence[old_ind] = quality;
	 old_ind = -1;
	 }
	 }
	 }
	 //System.out.println("#");
	 }
	 // */
	
	// check whether this alignment needs to be trimmed
	// and do so if necessary
	private static Integer[] adjust_alignment(Integer[] r, int pos, int read_len) {
		if (r.length == 1) {
			// st.get(i) is an index into that segment
			// do nothing, this is as simple as it gets
		}
		else {
			// find first segment used
			int j=0;
			Segment temp_1 = s.getSegment(r[0]);
			if (pos >= Math.min(temp_1.length(),fasta_readlen-1)) {
				pos -= Math.min(temp_1.length(),fasta_readlen-1);
				j++;
				temp_1 = s.getSegment(r[j]);
				while (pos >= temp_1.length() && temp_1.length() > 0) {
					if (temp_1.length() > 0) {
						pos -= Math.min(temp_1.length(),fasta_readlen-1);
						j++;
						temp_1 = s.getSegment(r[j]);
					}
					else {
						pos = 0;
					}
				}
			}
			
			// pos is now an index into the jth segment in r
			// (subsegment if j=0)
			int total = 0;
			if (j == 0) {
				total += Math.min(s.getSegment(r[0]).length(),fasta_readlen-1)-pos;
			}
			else {
				total += s.getSegment(r[j]).length()-pos;
			}
			int k=j;
			if (total >= read_len || s.getSegment(r[j]).length() == 0) {
				// then j is the only relevant segment
			}
			else {
				// let's find the end!
				while (total < read_len) {
					k++;
					temp_1 = s.getSegment(r[k]);
					if (temp_1.length() > 0) {
						total += temp_1.length();
					}
					else {
						// we've arrived at the poly-A tail!
						total = read_len;
					}
				}
			}
			
			// trim r between j and k
			Integer[] temp = new Integer[k-j+1];
			int ind = 0;
			for (int x=j; x<=k; x++) {
				temp[ind] = r[x];
				ind++;
			}
			r = temp;
		}
		return r;
	}
}