import java.util.*;

public class JunctionRead {

	// test method: calculates edge weights through simple counts
    // problem: this counts everything that starts in 3
    public static void edgecount(ArrayList[] alignment, SpliceGraph s) {
		Edge[] edge_list = s.getEdgeList();
		Edge[][] edges = s.getEdges();  // we'll pull num_segments from this
		double[] counts = new double[edge_list.length];
		double[] totals = new double[edges.length]; // outgoing edge counts
		
		for (int i=0; i<alignment.length; i++) {
			for (int j=0; j<alignment[i].size(); j++) {
				Integer[] r = (Integer[]) alignment[i].get(j);
				if (r.length > 1) {
					// we have an intron
					for (int k=0; k<r.length-1; k++) {
						Edge e = edges[r[k].intValue()][r[k+1].intValue()];
						int index = s.getEdgeIndex(e);
						counts[index]++;
						totals[r[k].intValue()]++;
					}
				}
				// initial edge estimation; may not be necessary
				if (edges[0][r[0].intValue()] != null) {
					int index = s.getEdgeIndex(edges[0][r[0].intValue()]);
					counts[index]++;
				}
			}
		}
		
		// add a pseudocount
		for (int i=0; i<counts.length; i++) {
			if (s.get_outdegree(edge_list[i]) > 1) {
				counts[i]++;
				totals[s.getSegIndex(edge_list[i].source)]++;
			}
		}
		
		// also may not be necessary
		for (int i=0; i<edges.length; i++) {
			if (edges[0][i] != null) {
				int index = s.getEdgeIndex(edges[0][i]);
				double length = s.getSegment(i).length();
				counts[index] /= length;
				totals[0] += counts[index];
			}
		}
		
		for (int i=0; i<edge_list.length; i++) {
			int source = s.getSegIndex(edge_list[i].source);
			// update if we have something, otherwise leave it alone
			if (totals[source] > 0) {
				edge_list[i].weight = counts[i]/(double) totals[source];
			}
		}
    }
	
	public static void edgecount(String[] reads, SpliceGraph s) {
		ArrayList[] alignment = s.readSubPaths(reads);
		Edge[] edge_list = s.getEdgeList();
		Edge[][] edges = s.getEdges();  // we'll pull num_segments from this
		double[] counts = new double[edge_list.length];
		double[] totals = new double[edges.length]; // outgoing edge counts
		
		for (int i=0; i<alignment.length; i++) {
			for (int j=0; j<alignment[i].size(); j++) {
				Integer[] r = (Integer[]) alignment[i].get(j);
				if (r.length > 1) {
					// we have an intron
					for (int k=0; k<r.length-1; k++) {
						Edge e = edges[r[k].intValue()][r[k+1].intValue()];
						int index = s.getEdgeIndex(e);
						counts[index]++;
						totals[r[k].intValue()]++;
					}
				}
				// initial edge estimation; may not be necessary
				if (edges[0][r[0].intValue()] != null) {
					int index = s.getEdgeIndex(edges[0][r[0].intValue()]);
					counts[index]++;
				}
			}
		}
		
		// add a pseudocount
		for (int i=0; i<counts.length; i++) {
			if (s.get_outdegree(edge_list[i]) > 1) {
				counts[i]++;
				totals[s.getSegIndex(edge_list[i].source)]++;
			}
		}
		
		// also may not be necessary
		for (int i=0; i<edges.length; i++) {
			if (edges[0][i] != null) {
				int index = s.getEdgeIndex(edges[0][i]);
				double length = s.getSegment(i).length();
				counts[index] /= length;
				totals[0] += counts[index];
			}
		}
		
		for (int i=0; i<edge_list.length; i++) {
			int source = s.getSegIndex(edge_list[i].source);
			// update if we have something, otherwise leave it alone
			if (totals[source] > 0) {
				edge_list[i].weight = counts[i]/(double) totals[source];
			}
		}
    }

}