import java.io.*;
import java.util.*;

public class LLComp {
    public static void main(String[] args) {
	if (args.length < 2) {
	    System.out.println("Usage: LLComp readfile psg1 psg2...");
	    System.exit(0);
	}

	SpliceGraph[] graphs = new SpliceGraph[args.length-1];
	for (int i=1; i<args.length; i++) {
	    graphs[i-1] = new SpliceGraph(args[i]);
	}

	String readfile = args[0];
	String[] reads_1 = null;
	ArrayList<String> lines = new ArrayList<String>();
	try {
	    Scanner r = new Scanner(new File(readfile));
	    while (r.hasNextLine()) {
		String line = r.nextLine();
		lines.add(line);
	    }
	    reads_1 = Utility.StringArray(lines);
	}
	catch (IOException ioe) {
	    System.out.println(readfile+" contains an error.");
	    System.exit(0);
	}

	for (int i=0; i<graphs.length; i++) {
	    System.out.print("Log likelihood (");
	    System.out.print(args[i+1]+"): ");
	    System.out.println(graphs[i].getLogLikelihood(reads_1));
	}
    }

    // calculates Euclidean distance between two vectors
    private static double euclid(double[] a, double[] b) {
	double result = 0;
	for (int i=0; i<a.length; i++) {
	    result += Math.pow(a[i]-b[i],2);
	}
	return Math.sqrt(result);
    }
    
    // calculates Manhattan distance between two vectors
    private static double manhattan(double[] a, double[] b) {
	double result = 0;
	for (int i=0; i<a.length; i++) {
	    result += Math.abs(a[i]-b[i]);
	}
	return result;
    }
}