import java.io.*;
import java.util.*;

public class LearnStructure {
    public static void main(String[] args) {
	if (args.length != 2) {
	    System.out.println("Usage: genRead psg reads");
	    System.exit(0);
	}

	String psgfile = args[0];
	String readfile = args[1];

	SpliceGraph s = new SpliceGraph(psgfile);
	s.printSpliceGraph();

	ArrayList<String> lines = new ArrayList<String>();
	try {
	    Scanner r = new Scanner(new File(readfile));
	    while (r.hasNextLine()) {
		String line = r.nextLine();
		lines.add(line);
	    }
	}
	catch (IOException ioe) {
	    System.out.println(readfile+" contains an error.");
	    System.exit(0);
	}
	String[] reads = Utility.StringArray(lines);
	System.out.println("Initial ll: "+s.getLogLikelihood(reads));
	s.resetEdgeWeights();
	double initial_BIC = s.learnEM(reads,1);
	System.out.println("BIC: "+initial_BIC);
	SpliceGraph base = new SpliceGraph(s);
	SpliceGraph best = new SpliceGraph(s);

	int nextDup = s.nextDup(0);
	while (nextDup < s.getNumSegments()) {
	    s.duplicateSegment(nextDup);
	    s.resetEdgeWeights();
	    double BIC = s.learnEM(reads,1);
	    s.printSpliceGraph();
	    if (BIC < initial_BIC) {
		nextDup = s.nextDup(nextDup);
		s = new SpliceGraph(best);
	    }
	    else {
		best = new SpliceGraph(s);
		nextDup = s.nextDup(0);
	    }
	    System.out.println("new BIC: "+BIC);
	}
	//s.printSpliceGraph();
	//s.printEdgeMatrix();
	//s.resetEdgeWeights();
	//BIC = s.learnEM(reads,1);
	//System.out.println("BIC: "+BIC);
    }

    // calculates Euclidean distance between two vectors
    private static double euclid(double[] a, double[] b) {
	double result = 0;
	for (int i=0; i<a.length; i++) {
	    result += Math.pow(a[i]-b[i],2);
	}
	return Math.sqrt(result);
    }
    
    // calculates Manhattan distance between two vectors
    private static double manhattan(double[] a, double[] b) {
	double result = 0;
	for (int i=0; i<a.length; i++) {
	    result += Math.abs(a[i]-b[i]);
	}
	return result;
    }
}