import java.io.*;
import java.util.ArrayList;
import java.util.Stack;

public class MakeFasta {
	public static void main(String[] args) {
		if (args.length != 2) {
			System.out.println("usage: java MakeFasta <psg> <readlen>");
			System.exit(0);
		}
		
		String filename = args[0];
		int read_len = Integer.parseInt(args[1]);
		SpliceGraph s = new SpliceGraph(filename);
		String base = filename.substring(0,filename.indexOf('.'));
		base += ".fa";
		
		Segment[] segments = s.getSegments();
		Edge[][] edges = s.getEdges();
		//String seq = s.getSequence();
		
		try {
			BufferedWriter out = new BufferedWriter(new FileWriter(base));
			for (int j=0; j<segments.length-1; j++) {
				if (segments[j].length() > 0) {
					
					// SINGLE SEGMENT ALIGNMENT
					String compare = segments[j].sequence;
					out.write(">s"+j);
					out.newLine();
					out.write(compare);
					out.newLine();
					
					// MULTIPLE SEGMENT ALIGNMENT
					Stack<Segment> stack = new Stack<Segment>();
					int total = 0;
					ArrayList<Integer> alignment = new ArrayList<Integer>();
					stack.push(segments[j]); 	  // added to stack
					alignment.add(j);             // marked
					if (read_len-1 >= segments[j].length()) {
						compare = segments[j].sequence;
					}
					else {
						int sub_ind = segments[j].length()-read_len+1;
						compare = segments[j].sequence.substring(sub_ind);
					}
					int initial_len = compare.length();
					// initialize stack
					for (int k=0; k<edges[j].length; k++) {
						if (edges[j][k] != null) {
							stack.push(segments[k]);
						}
					}
					
					while (!stack.empty()) {
						Integer stacktop = s.getSegIndex(stack.peek());
						if (alignment.contains(stacktop)) {
							// marked: pop
							int cut = Math.max(compare.length()-stack.peek().length(), 0);
							compare = compare.substring(0,cut);
							total = compare.length()-initial_len;
							stack.pop();
							alignment.remove(alignment.size()-1);
						}
						else if (stack.peek().length()>=(read_len-total-1)
								 || stacktop.intValue()==segments.length-1) {
							// long enough: compare then pop
							alignment.add(stacktop);
							int end = read_len-total-1;
							int cut = compare.length();
							if (stacktop.intValue() < segments.length-1) {
								compare += stack.peek().sequence.substring(0,end);
							}
							else {
								for (int k=0; k<read_len-total-1; k++) {
									compare += "A";
								}
							}
							out.write(">s");
							for (int i=0; i<alignment.size(); i++) {
								out.write(alignment.get(i).toString());
								if (i<alignment.size()-1) {
									out.write(":");
								}
							}
							out.newLine();
							out.write(compare);
							out.newLine();
							compare = compare.substring(0,cut);
							stack.pop();
							alignment.remove(alignment.size()-1);
						}
						else {
							// not long enough: add seq, push adjacent segments
							alignment.add(stacktop);
							int ind = stacktop.intValue();
							compare += stack.peek().sequence;
							total += stack.peek().length();
							for (int k=0; k<edges[ind].length; k++) {
								if (edges[ind][k] != null) {
									stack.push(segments[k]);
								}
							}
						}
					}
				}
			}
			out.close();
		}
		catch (IOException ioe) {
			System.exit(0);
		}
	}
}