import java.util.*;
import java.io.*;

public class MakeRScript {
	
    private static double[][] real_em_means;
    private static double[][] real_jr_means;
    private static double[][] real_em_sd;
    private static double[][] real_jr_sd;
    
    private static double[][] sim_em_means;
    private static double[][] sim_jr_means;
    private static double[][] sim_em_sd;
    private static double[][] sim_jr_sd;
	private static double[] true_weights;
	
	private static final int NUM_TRIALS = 4;
	
    public static void main(String[] args) {
		if (args.length != 2) {
			System.out.println("usage: MakeRScript <sim.out> <real.out>");
			System.exit(0);
		}
		
		String simfile = args[0];
		String realfile = args[1];
		
		if (!simfile.substring(simfile.lastIndexOf("_")).equals("_sim.out") ||
			!realfile.substring(realfile.lastIndexOf("_")).equals("_real.out")) {
			System.out.println("usage: MakeRScript <sim.out> <real.out>");
			System.exit(0);
		}
		
		String gene_name = simfile.substring(0,simfile.lastIndexOf("_"));
		
		try {
			process_files(simfile, realfile);
			write_script(gene_name);
		}
		catch (IOException ioe) {
			System.out.println("Error processing files.");
			System.exit(0);
		}
    }
    
	// writes R script
    private static void write_script(String gene_name) throws IOException {
		BufferedWriter out = new BufferedWriter(new FileWriter("script.R"));
		// let's do four plots per row (arbitrary choice)
		int num_rows = sim_em_means.length/4;
		if (sim_em_means.length%4 != 0) {
		    num_rows++;
		}
		out.write("pdf(\"");
		out.write(gene_name);
		out.write("_sim.pdf\")");
		out.newLine();
		out.write("library(gplots)");
		out.newLine();
		out.write("par(mfrow=c("+num_rows+",2))");
		out.newLine();
		
		for (int i=0; i<sim_em_means.length; i++) {
			// write each individual plot for simulated data
			out.write("means.em <- c(");
			for (int j=0; j<sim_em_means[i].length; j++) {
				out.write(Double.toString(sim_em_means[i][j]));
				if (j < sim_em_means[i].length-1) {
					out.write(",");
				}
			}
			out.write(")");
			out.newLine();
			out.write("means.jr <- c(");
			for (int j=0; j<sim_jr_means[i].length; j++) {
				out.write(Double.toString(sim_jr_means[i][j]));
				if (j < sim_jr_means[i].length-1) {
					out.write(",");
				}
			}
			out.write(")");
			out.newLine();
			out.write("sd.em <- c(");
			for (int j=0; j<sim_em_sd[i].length; j++) {
				out.write(Double.toString(sim_em_sd[i][j]));
				if (j < sim_em_sd[i].length-1) {
					out.write(",");
				}
			}
			out.write(")");
			out.newLine();
			out.write("sd.jr <- c(");
			for (int j=0; j<sim_jr_sd[i].length; j++) {
				out.write(Double.toString(sim_jr_sd[i][j]));
				if (j < sim_jr_sd[i].length-1) {
					out.write(",");
				}
			}
			out.write(")");
			out.newLine();
			out.write("plotCI(x=means.em, uiw=sd.em, lty=2, xlim=c(0.5,"+NUM_TRIALS+".5), ylim=c(0,1), gap=0,");
			out.write(" ylab=\"Edge Weight\", xlab=\"Size of Read Set\", col=\"red\")");
			out.newLine();
			out.write("plotCI(x=means.jr, uiw=sd.jr, lty=1, xlim=c(0.5,"+NUM_TRIALS+".5), ylim=c(0,1), gap=0, add=TRUE)");
			out.newLine();
			out.write("abline(h="+true_weights[i]+", col=4, lty=2)");
			out.newLine();
		}
		out.write("dev.off()");
		out.newLine();
		out.write("pdf(\"");
		out.write(gene_name);
		out.write("_real.pdf\")");
		out.newLine();
		out.write("par(mfrow=c("+num_rows+",2))");
		out.newLine();
		for (int i=0; i<real_em_means.length; i++) {
			// write each individual plot for real data
			out.write("means.em <- c(");
			for (int j=0; j<real_em_means[i].length; j++) {
				out.write(Double.toString(real_em_means[i][j]));
				if (j < real_em_means[i].length-1) {
					out.write(",");
				}
			}
			out.write(")");
			out.newLine();
			out.write("means.jr <- c(");
			for (int j=0; j<real_jr_means[i].length; j++) {
				out.write(Double.toString(real_jr_means[i][j]));
				if (j < real_jr_means[i].length-1) {
					out.write(",");
				}
			}
			out.write(")");
			out.newLine();
			out.write("sd.em <- c(");
			for (int j=0; j<real_em_sd[i].length; j++) {
				out.write(Double.toString(real_em_sd[i][j]));
				if (j < real_em_sd[i].length-1) {
					out.write(",");
				}
			}
			out.write(")");
			out.newLine();
			out.write("sd.jr <- c(");
			for (int j=0; j<real_jr_sd[i].length; j++) {
				out.write(Double.toString(real_jr_sd[i][j]));
				if (j < real_jr_sd[i].length-1) {
					out.write(",");
				}
			}
			out.write(")");
			out.newLine();
			out.write("plotCI(x=means.em, uiw=sd.em, lty=2, xlim=c(0.5,"+NUM_TRIALS+".5), ylim=c(0,1), gap=0,");
			out.write(" ylab=\"Edge Weight\", xlab=\"Size of Read Set\", col=\"red\")");
			out.newLine();
			out.write("plotCI(x=means.jr, uiw=sd.jr, lty=1, xlim=c(0.5,"+NUM_TRIALS+".5), ylim=c(0,1), gap=0, add=TRUE)");
			out.newLine();
		}
		out.write("dev.off()");
		out.newLine();
		out.close();
	}
	
	// format of output:
	//		i = edge
	//		j = number of reads
	private static void process_files(String sim, String real) throws IOException {
		// stuff we need to read files
		Scanner s1 = new Scanner(new File(sim));
		Scanner s2 = new Scanner(new File(real));
		boolean isEM = false;
		boolean isMeans = false;
		
		int num_edges = 0;
		int num_not1 = 0;
		
		// first, need to count number of lines.
		Scanner in = new Scanner(new File(sim));
		while (in.hasNextLine()) {
			String line = in.nextLine();
			if (line.trim().equals("Means:")) {
				// we've started a new means section
				isMeans = true;
				if (!isEM) {
					isEM = true;
				}
				else {
					isEM = false;
				}
			}
			else if (line.trim().equals("Variances:")) {
				// we've started a new standard deviation section
				break;
			}
			else {
				// we have data
				if (isEM && isMeans) {
					num_edges++;
					String[] components = line.split("\t");
					if (Double.parseDouble(components[1]) != 1.0) {
						num_not1++;
					}
				}
			}
		}
		System.out.println("Found "+num_edges+" edges, "+num_not1+" of which were not weighted 1.0");
		
		// reinitialize the scanner and initialize the data storage arrays
		s1 = new Scanner(new File(sim));
		s1.nextLine(); // clear the buffer
		s2.nextLine(); // same
		isEM = false;
		isMeans = false;
		
		real_em_means = new double[num_not1][NUM_TRIALS];
		real_jr_means = new double[num_not1][NUM_TRIALS];
		real_em_sd = new double[num_not1][NUM_TRIALS];
		real_jr_sd = new double[num_not1][NUM_TRIALS];
		
		sim_em_means = new double[num_not1][NUM_TRIALS];
		sim_jr_means = new double[num_not1][NUM_TRIALS];
		sim_em_sd = new double[num_not1][NUM_TRIALS];
		sim_jr_sd = new double[num_not1][NUM_TRIALS];
		true_weights = new double[num_not1];
		
		// now, pull the data from the files
		int index=0;
		int edge_num=0;
		while (s1.hasNextLine()) {
			String line = s1.nextLine();
			if (line.trim().equals("Means:")) {
				// we've started a new means section
				index = 0;
				isMeans = true;
				if (!isEM) {
					isEM = true;
				}
				else {
					isEM = false;
				}
			}
			else if (line.trim().equals("Variances:")) {
				// we've started a new standard deviation section
				index = 0;
				isMeans = false;
			}
			else {
				// we have data
				if (isEM) {
					if (isMeans) {
						String[] components = line.split("\t");
						if (Double.parseDouble(components[1]) != 1.0) {
							// store the data
							for (int i=1; i<=NUM_TRIALS; i++) {
								sim_em_means[index][i-1] = Double.parseDouble(components[i]);
							}
							true_weights[index] = Double.parseDouble(components[NUM_TRIALS+1]);
							index++;
						}
						edge_num++;
					}
					else {
						String[] components = line.split("\t");
						if (Double.parseDouble(components[2]) != 0.0) {
							// store the data
							for (int i=1; i<=NUM_TRIALS; i++) {
								sim_em_sd[index][i-1] = Double.parseDouble(components[i]);
							}
							index++;
						}
					}
				}
				else {
					if (isMeans) {
						String[] components = line.split("\t");
						if (Double.parseDouble(components[1]) != 1.0) {
							// store the data
							for (int i=1; i<=NUM_TRIALS; i++) {
								sim_jr_means[index][i-1] = Double.parseDouble(components[i]);
							}
							index++;
						}
					}
					else {
						String[] components = line.split("\t");
						if (Double.parseDouble(components[2]) != 0.0) {
							// store the data
							for (int i=1; i<=NUM_TRIALS; i++) {
								sim_jr_sd[index][i-1] = Double.parseDouble(components[i]);
							}
							index++;
						}
					}
				}
			}
		}
		while (s2.hasNextLine()) {
			String line = s2.nextLine();
			if (line.trim().equals("Means:")) {
				// we've started a new means section
				index = 0;
				isMeans = true;
				if (!isEM) {
					isEM = true;
				}
				else {
					isEM = false;
				}
			}
			else if (line.trim().equals("Variances:")) {
				// we've started a new standard deviation section
				index = 0;
				isMeans = false;
			}
			else {
				// we have data
				if (isEM) {
					if (isMeans) {
						String[] components = line.split("\t");
						if (Double.parseDouble(components[1]) != 1.0) {
							// store the data
							for (int i=1; i<=NUM_TRIALS; i++) {
								real_em_means[index][i-1] = Double.parseDouble(components[i]);
							}
							index++;
						}
					}
					else {
						String[] components = line.split("\t");
						if (Double.parseDouble(components[2]) != 0.0) {
							// store the data
							for (int i=1; i<=NUM_TRIALS; i++) {
								real_em_sd[index][i-1] = Double.parseDouble(components[i]);
							}
							index++;
						}
					}
				}
				else {
					if (isMeans) {
						String[] components = line.split("\t");
						if (Double.parseDouble(components[1]) != 1.0) {
							// store the data
							for (int i=1; i<=NUM_TRIALS; i++) {
								real_jr_means[index][i-1] = Double.parseDouble(components[i]);
							}
							index++;
						}
					}
					else {
						String[] components = line.split("\t");
						if (Double.parseDouble(components[2]) != 0.0) {
							// store the data
							for (int i=1; i<NUM_TRIALS; i++) {
								real_jr_sd[index][i-1] = Double.parseDouble(components[i]);
							}
							index++;
						}
					}
				}
			}
		}
		
	}
}