import java.io.*;

public class PSGTest {
    public static void main(String[] args) {
	if (args.length != 3 && args.length != 4) {
	    System.out.println("Usage: genRead [-v] psg readLen numTrial");
	    System.exit(0);
	}

	int verbose = 0;
	if (args.length == 4) { verbose = 1; }

	String filename = args[0];
	int howLong = Integer.parseInt(args[verbose+1]);
	int numTrial = Integer.parseInt(args[verbose+2]);

	SpliceGraph s = new SpliceGraph(filename);

	double[] initial = s.getEdgeWeights();
	double read_1_diff = 0, read_1_m = 0;
	double read_2_diff = 0, read_2_m = 0;
	double read_3_diff = 0, read_3_m = 0;
	double read_4_diff = 0, read_4_m = 0;
	double read_5_diff = 0, read_5_m = 0;
	double read_6_diff = 0, read_6_m = 0;
	// double read_7_diff = 0, read_7_m = 0;

	for (int i=0; i<numTrial; i++) {
	    s = new SpliceGraph(filename);
	    String[] reads_1 = s.generateReads(10,howLong);
	    String[] reads_2 = s.generateReads(100,howLong);
	    String[] reads_3 = s.generateReads(1000,howLong);
	    String[] reads_4 = s.generateReads(10000,howLong);
	    String[] reads_5 = s.generateReads(100000,howLong);
	    String[] reads_6 = s.generateReads(1000000,howLong);
	    // String[] reads_7 = s.generateReads(10000000,howLong);
	    s.resetEdgeWeights();
	    s.learnEM(reads_1,verbose);
	    read_1_diff += euclid(initial,s.getEdgeWeights());
	    read_1_m += manhattan(initial,s.getEdgeWeights());
	    if (verbose == 0) { System.out.print("."); }
	    s.resetEdgeWeights();
	    s.learnEM(reads_2,verbose);
	    read_2_diff += euclid(initial,s.getEdgeWeights());
	    read_2_m += manhattan(initial,s.getEdgeWeights());
	    if (verbose == 0) { System.out.print("."); }
	    s.resetEdgeWeights();
	    s.learnEM(reads_3,verbose);
	    read_3_diff += euclid(initial,s.getEdgeWeights());
	    read_3_m += manhattan(initial,s.getEdgeWeights());
	    if (verbose == 0) { System.out.print("."); }
	    s.resetEdgeWeights();
	    s.learnEM(reads_4,verbose);
	    read_4_diff += euclid(initial,s.getEdgeWeights());
	    read_4_m += manhattan(initial,s.getEdgeWeights());
	    if (verbose == 0) { System.out.print("."); }
	    s.resetEdgeWeights();
	    s.learnEM(reads_5,verbose);
	    read_5_diff += euclid(initial,s.getEdgeWeights());
	    read_5_m += manhattan(initial,s.getEdgeWeights());
	    if (verbose == 0) { System.out.print("."); }
	    s.learnEM(reads_6,verbose);
	    read_6_diff += euclid(initial,s.getEdgeWeights());
	    read_6_m += manhattan(initial,s.getEdgeWeights());
	    if (verbose == 0) { System.out.print("."); }
	    /*s.learnEM(reads_7,verbose);
	    read_7_diff += euclid(initial,s.getEdgeWeights());
	    read_7_m += manhattan(initial,s.getEdgeWeights());
	    if (verbose == 0) { System.out.print("."); }*/
	}
	System.out.println();

	read_1_diff /= numTrial;
	read_2_diff /= numTrial;
	read_3_diff /= numTrial;
	read_4_diff /= numTrial;
	read_5_diff /= numTrial;
	read_6_diff /= numTrial;
	// read_7_diff /= numTrial;
	read_1_m /= numTrial;
	read_2_m /= numTrial;
	read_3_m /= numTrial;
	read_4_m /= numTrial;
	read_5_m /= numTrial;
	read_6_m /= numTrial;
	// read_7_m /= numTrial;

	System.out.println("Results, averaged over "+numTrial+" runs:");
	System.out.println("     10 reads: "+read_1_diff+", "+read_1_m);
	System.out.println("    100 reads: "+read_2_diff+", "+read_2_m);
	System.out.println("   1000 reads: "+read_3_diff+", "+read_3_m);
	System.out.println("  10000 reads: "+read_4_diff+", "+read_4_m);
	System.out.println(" 100000 reads: "+read_5_diff+", "+read_5_m);
	System.out.println("     1M reads: "+read_6_diff+", "+read_6_m);
	// System.out.println("    10M reads: "+read_7_diff+", "+read_7_m);
    }

    // calculates Euclidean distance between two vectors
    private static double euclid(double[] a, double[] b) {
	double result = 0;
	for (int i=0; i<a.length; i++) {
	    result += Math.pow(a[i]-b[i],2);
	}
	return Math.sqrt(result);
    }
    
    // calculates Manhattan distance between two vectors
    private static double manhattan(double[] a, double[] b) {
	double result = 0;
	for (int i=0; i<a.length; i++) {
	    result += Math.abs(a[i]-b[i]);
	}
	return result;
    }
}