import java.io.*;

public class PSG_VLTest {
    public static void main(String[] args) {
	if (args.length != 3) {
	    System.out.println("Usage: genRead psg readLen numTrial");
	    System.exit(0);
	}

	String filename = args[0];
	int howLong = Integer.parseInt(args[1]);
	int numTrial = Integer.parseInt(args[2]);

	SpliceGraph s = new SpliceGraph(filename);

	double[] initial = s.getEdgeWeights();
	double[] read_5_wt = new double[initial.length];
	double[] read_6_wt = new double[initial.length];
	//double[] read_7_wt = new double[initial.length];
	double read_5_diff = 0, read_5_m = 0;
	double read_6_diff = 0, read_6_m = 0;
	//double read_7_diff = 0, read_7_m = 0;

	for (int i=0; i<numTrial; i++) {
	    s = new SpliceGraph(filename);
	    String[] reads_5 = s.generateReads(100000,howLong);
	    String[] reads_6 = s.generateReads(1000000,howLong);
	    //String[] reads_7 = s.generateReads(10000000,howLong);
	    s.resetEdgeWeights();
	    s.learnEM(reads_5,0);
	    double[] temp = s.getEdgeWeights();
	    for (int j=0; j<temp.length; j++) {
		read_5_wt[j] += temp[j];
	    }
	    read_5_diff += euclid(initial,temp);
	    read_5_m += manhattan(initial,temp);
	    System.out.print(".");
	    s.resetEdgeWeights();
	    s.learnEM(reads_6,0);
	    temp = s.getEdgeWeights();
	    for (int j=0; j<temp.length; j++) {
		read_6_wt[j] += temp[j];
	    }
	    read_6_diff += euclid(initial,temp);
	    read_6_m += manhattan(initial,temp);
	    System.out.print(".");
	    /*s.resetEdgeWeights();
	    s.learnEM(reads_7,0);
	    temp = s.getEdgeWeights();
	    for (int j=0; j<temp.length; j++) {
		read_7_wt[j] += temp[j];
	    }
	    read_7_diff += euclid(initial,temp);
	    read_7_m += manhattan(initial,temp);
	    System.out.print(".");*/
	}
	System.out.println();

	for (int i=0; i<read_5_wt.length; i++) {
	    read_5_wt[i] /= numTrial;
	    read_6_wt[i] /= numTrial;
	}
	
	read_5_diff /= numTrial;
	read_6_diff /= numTrial;
	read_5_m /= numTrial;
	read_6_m /= numTrial;

	System.out.println("Results, averaged over "+numTrial+" runs:");
	System.out.println(" 100000 reads: "+read_5_diff+", "+read_5_m);
	System.out.println("     1M reads: "+read_6_diff+", "+read_6_m);
	System.out.println();
	System.out.print("100000 reads ::");
	for (int i=0; i<read_5_wt.length; i++) {
	    System.out.printf(" %.2f",read_5_wt[i]);
	}
	System.out.println();
	System.out.print("   1MM reads ::");
	for (int i=0; i<read_6_wt.length; i++) {
	    System.out.printf(" %.2f",read_6_wt[i]);
	}
	System.out.println();
    }

    // calculates Euclidean distance between two vectors
    private static double euclid(double[] a, double[] b) {
	double result = 0;
	for (int i=0; i<a.length; i++) {
	    result += Math.pow(a[i]-b[i],2);
	}
	return Math.sqrt(result);
    }
    
    // calculates Manhattan distance between two vectors
    private static double manhattan(double[] a, double[] b) {
	double result = 0;
	for (int i=0; i<a.length; i++) {
	    result += Math.abs(a[i]-b[i]);
	}
	return result;
    }
}